# Design

## ドキュメント

### ソーステキストファースト

ドキュメントの可読性の基準はソーステキスト(Markdownテキスト)でありレンダリング後の可読性はオプションである。
よって構文はレンダリングを要さず直接意味付けの役割を果たさなければならない。

ゆえにすべての構文は直感に従い意味を解釈できる。

### ドキュメント指向

ドキュメントはHTMLドキュメントに要求される人的操作性を損なってはならない。
よって構文は次の操作的機能を備えなければならない。

- 参照性
  - 主要項目を識別子により参照可能である
- 編集性
  - 主要項目の識別子を任意の識別子で固定可能である
  - 識別子は記述位置に依存せず可搬である

ゆえに構文は識別子の設定および参照構文を備える。

### 全文展開

ドキュメントはその全文の閲覧のために追加の操作を必要としてはならない。
よってドキュメントはその全文を展開して表示しなければならない。

ゆえに構文はレンダリング後の操作により表示する内容および状態を変化させる構文を備えない。

### モノクロプリント対応

構文はモノクロ表示および印刷に対応していなければならない。

ゆえに構文はCSSにより次の通り表示を変更することで意味付けを維持したままモノクロ版に変換できる。

|Syntax|HTML   |Presentation
|------|-------|------------
| ==   |\<mark>|下線(通常、幅広、または波型)
| ++   |\<ins> |下線(幅広または波型)
| ~~   |\<del> |打ち消し線

### 閲覧者によるマークアップ方法の確保

構文およびその表現は閲覧者による基本的なマークアップを妨げてはならない。

ゆえに構文は下線、打ち消し線のための構文を備えない。
ハイライトは閲覧者によるハイライトを妨げないよう線幅が40%以下かつ淡色でなければならない。
閲覧者によるマークアップはベースがカラーの場合は下線、モノクロの場合はモノクロ版に変換のうえマーカーにより行うことを企図している。

### ユニバーサルポータビリティ

ドキュメントは異なるサイトやツール上でも一貫して表示されなければならない。
よってパーサーの構成は固定でなければならない。
またリンクは固定できなければならない。

ゆえにAPIはパーサーの構成を変更できない。
また非従属コンテンツへの相対リンクはドメインを固定できる。

### 自己完結性

ドキュメントはすべて初期(共通)構成の閲覧環境で閲覧できなければならない。
よって閲覧に拡張機能等の追加が必要となってはならない

ゆえにドキュメントはサードパーティツールなど自己完結性を損なうツールの使用をサポートしない。
また自己完結性を損なうエコシステムを構築しない。

### サードパーティドキュメントの共有および表示可能性

ドキュメントはそのセキュリティ上の信頼性にかかわらず共有および表示できなければならない。
よってパーサーおよびレンダラーはXSSをはじめとする悪意あるコードの注入に対して安全でなければならない。

#### XSS安全

パーサーはパース結果をオブジェクトとして生成しHTML等の文字列での表現を経由しない。

ゆえにパーサーはXSS安全である。
ただし外部メディアの表示等安全性を外部サービスに依存する部分はこの限りではない。

#### Fishing安全

パーサーはリンクテキストとリンク先の一致しないリンクを生成しない。

ゆえにパーサーはFishing安全である。

#### DOS安全

パーサーは入力文字列により実行環境を応答不能にすることはない。

ゆえにパーサーはDOS安全である。

##### インプットサイズ

パーサーは上限を超えるサイズの入力をパースしない。
また上限を超えない入力は1秒以下でセグメントに分割され非同期での逐次パースが可能である。

ゆえに入力はパーサーの実行環境を応答不能にすることはない。

##### セグメントサイズ

パーサーは上限を超えるサイズのセグメントをパースしない。
また上限を超えないセグメントのパースも規定のパースコストの上限を超えない。

ゆえに入力はパーサーの実行環境を応答不能にすることはない。
なお1セグメントは1ブロックに対応する。

##### パースコスト

パーサーはパースコストが上限を超えるセグメントをパースしない。
パースコストは同一セグメント内で内部セグメントの境界を超えて共有されるため内部セグメントが存在する場合もこの制約は保たれる。

ゆえにセグメントは過大な入力または過剰なバックトラックによりパーサーの実行環境を応答不能にすることはない。
なおこの上限は非常に高く設定されており通常の使用で上限を超えることは考えにくい。

### 情報通信フォーマット

以上のようにSecuremarkは(ドキュメント形式、すなわちインテリジェンス化された)情報の伝達および共有を要件として設計された情報通信フォーマットである。
よってこの能力を失わせる変更を行ってはならない。

## 構文

### 引用文可搬性

引用構文はコピー＆ペーストにより挿入されたテキストを無編集で使用できなければならない。

ゆえに引用構文は引用文をパースしない。

### 改行不変性

改行は表示文字列とこれを入力とする再出力で不変でなければならない。

ゆえに改行は入力と出力で一貫することを標準とする。

### オートリンク不変性

オートリンクは表示文字列とこれを入力とする再出力において解釈が不変でなければならない。

ゆえにハッシュタグ構文は構文が表示文字列と一致する構文であるオートリンク構文(`#tag`)としてのみ構文化され、表示文字列から構文の範囲が特定不可能となる通常構文の形態(`#{tag}`)を持たない。

### 自動附番

ドキュメント内の参照はドキュメントの編集を妨げないよう参照先および参照元いずれも記述順序に非依存でなければならない。

ゆえに参照箇所に実体を記述する注釈構文および識別に文字列を使用する図表構文を採用し、その表示方法は任意とする。
脚注構文は附番が手作業となり参照と実体の対応の管理が困難であるため不採用とし注釈構文により生成可能とする。

### アクセント設定

ドキュメントは漫然と読まれないよう緩急をつけられなければならない。
よって構文は他の構文に続けて漫然と読み流せない異質な視覚表現を持ちアクセントとなる構文を備えなければならない。
なおこのためアクセント構文は他の構文と完全に統一されたデザインであってはならず若干異質なデザインでなければならない。

ゆえに構文は着色およびデザインにより視覚的刺激を与えるMessage構文を備える。
またAside構文は関心の変化により認知的刺激を与える。

### 羅列的知識への非依存

構文はその使用のために羅列的知識を求めてはならない。

ゆえにバックスラッシュエスケープの特殊処理の対象は改行文字のみに限られる。
またURL構文はHTMLエンティティの参照の有効性を検査しない。
HTMLEntity構文はエンティティ追加時の互換性確保のため不正なエンティティを検出する。

### テンプレート対応

ソーステキストは自身をテンプレートとして別のソーステキストを生成するための構文を使用できなければならない。
またテンプレートとなるソーステキストもドキュメントとして正しく表示できなければならない。
よって構文はテンプレートのソーステキストとドキュメント間での表示上の一貫性のための構文を備えなければならない。

ゆえに構文はテンプレート構文を備える。

### 編集中のソーステキストのレンダリング結果の表示安定性

レンダリング結果の表示はソーステキストの編集にともなう高頻度または広範囲にわたる変化により視覚的負担をかけてはならない。
よって構文は編集過程で生じる不完全な構文を最大限補完してパースしなければならない。

ゆえにすべてのブロック構文は順次的編集過程で生じるすべての不完全な構文を補完してパースする。
またインラインのレンダリングに含まれる高価な処理はキャッシュにより即時化される。

### 構文階層独立性

上位区分の構文の一致は下位区分の構文から独立して上位区分の構文の形式のみから判定されなければならない。
よって上位区分の構文の一致判定に下位区分の構文による内包文字列のパース結果を用いてはならない。

ゆえにすべてのブロック構文は構文の一致判定にインライン構文による内包文字列のパース結果を用いない。
またすべての構文は任意の包含関係において無効化および有効化できる。

### 構文不変性

任意のソーステキストの中で構文に一致する文字列は構文の拡張の前後において不変でなければならず追加の前後において最大限不変でなければならない。
よって構文の属性等のパラメータは値でなく書式で制約しなければならない。
また不正なパラメータまたは構成の文字列は誤入力および出力の将来的可変性を通知するパース結果を出力しなければならない。

ゆえにすべての構文はこのように実装される。

### 構文追加要件

構文の追加は原則として次のいずれかの要求に対応するものでなければならない。

- HTML上の情報表示形式への対応(\<ul>など)
- HTML上で固有の視覚表現を持つ意味付けへの対応(\<mark>など)
- HTMLドキュメントに要求される操作的機能の充足(Index構文など)

よって外観や制御のために構文を追加してはならない。
また他の構文と同じ視覚表現で異なる意味付けを行うHTMLタグを追加してはならない。
ディレクティブなど文書でなくページを記述するための構文を追加してはならない。

## スケーラビリティ

パーサーおよびレンダラーはドキュメントサイズの増大により以下の利用性を損なってはならない。

### 閲覧性

ドキュメントは即時ないし順次閲覧できなければならない。
よってドキュメントは全体のパースを待たず一定時間内に表示を開始しなければならない。
また高価な処理は必要時まで遅延しなければならない。

ゆえにパーサーはパースおよびその結果の返却を逐次的に行う。
またパーサーは外部リソースのリクエストその他の高価な処理を自身で行わずレンダラーに委ねる。

### 参照性

ドキュメントは各要素への参照、参照先への移動、および参照元への復帰方法を提供しなければならない。

ゆえにドキュメントはすべてのセクション、用語定義、図表、注釈、および出典をリンクを介して参照、移動、および復帰できる。

### 編集性

ドキュメントはそのサイズにかかわらず編集が容易でなければならない。
よってパースおよびレンダリングは差分更新により行わなければならない。

ゆえにパーサーは差分更新を行い、構文はこのために空行によるブロックの分割を必須とする。

## 記号解釈

ソーステキストを直接読む際の記号解釈。

### \#

定義付き語句。

### ~

同値的関係。

### :

説明。

### ^

参照識別子。

### !

解釈指示。

### $

数学的構文。

### [ ]

指示対象指定(構文の指示部分の括弧が後続する場合)。
対象は必ず可視の文字または要素から始まる。
対象部分(角括弧)は常に指示部分(波または丸括弧)に先行する。

## 備考

### 実行速度

ブラウザの以下の欠陥および低速性により速度低下が生じており今後のブラウザの改善が求められる。

- IntersectionObserverが描画完了前に交差を計算するためすべて同時発火する欠陥
- CSSカウンター(オーダードリスト)

### AST(CST)

Securemarkはブロック単位の差分更新によりリアルタイムレンダリングを高速化しているがCSTを生成する場合行の追加削除により以降すべてのノードの位置情報を更新する必要が生じ許容範囲を超える遅延が生じる可能性がある。このためこの最適化を行う必要があるが行位置の更新のみであればこれのみを行う処理経路の追加は容易であろう。なお描画に必要な差分情報はCSTから再計算させず別途提供により高速化すべきである。この際CSTの位置情報も静的値でなく参照の都度動的に再計算するよう変更しノードの再生成を要さず位置情報を更新できるよう最適化することが望ましい。これらは数百行以下に規律されるソースコードにおいては生じない問題であり数十万文字を許容しなければならないマークアップ言語に新出の問題であるため新規にCSTの設計および運用を再考する必要がある。

またBlockquote構文を内包テキストまでシンタックスハイライトする場合、構文(>)と内包テキストが垂直に分割されるため構文と1対1で対応する単純なノードではCSTを表現できず構文の断片を表現するノードが必要となり、さらにこれをエディタ上でシンタックスハイライトするためのHTMLに変換する際に各行においてこの断片のCSTを断片のHTMLに正しく変換および表示しなければならずかなり煩雑な作業を要する。

CodeMirrorが素では速いがVimModeでは数万文字程度でも耐え難く非常に遅いので高速な専用エディタの開発が必要。当面は数万文字以上のテキストではTextareaに切り替えることとする。

### バックトラック

SecuremarkのAnnotation構文に典型的であるように文脈を変更する構文の中にその文脈に依存し変更される他の構文が存在する場合一般に文脈の相違から解析結果を再利用不能な(`αA'β | αAB`)バックトラックが生じる。文脈依存構文の解析中に文脈により解釈の異なる字句(`[a((`)または現在の文脈を終了する字句(ただの括弧がスコープを作らない場合の`(([a))`)に遭遇したとき、現在の文脈での解析の継続を試行する解析方法はメモ化なしではバックトラックが再帰的に生じ線形時間で解析できず、現在の文脈での解析を中止し直前の文脈を継続または新たな文脈を開始する解析方法はバックトラックなしで線形時間で解析できる。
CommonMarkはLink構文から明らかなように後者の解析方法により再帰的バックトラックなし(Markdown(2n)＋正規表現(1n)で最悪計算量3n程度)で解析できる言語であるが同時にこれは文脈依存構文の正当な入れ子表現を解析できず文脈依存構文の存在する数に応じて多項式的(おそらく$n^{2+c}$)に構文の壊れやすさの増す脆く拡張性の低い言語であることを意味する。この問題はおそらくメモ化により解決できるがCommonMarkは実行性能追及のためメモ化を廃止しているためメモ化により性能を低下させてまで解決するつもりはないと思われる(すなわちCommonMarkは機械を至上とし人間に制約を課す低水準の言語であり人間の需要を至上とするSecuremarkとは対極に位置する)。
従って現在の再帰的バックトラックなしで解析可能な文法に制約されるCommonMarkには文脈依存構文を入れ子表現の広範な制限ならびに構文の可読性および開始記号の信頼性の多項式的な低下と引き換えにしか追加できないという拡張性の欠陥が存在する。CommonMarkの仕様策定者が構文の拡張に(名称を維持するか否かにかかわらず)不自然なまでに消極的または進展がないのは正当な理由や怠慢からでなく文脈依存構文を追加するにつれて構文解析戦略の欠陥が明白になっていくためおよび現在の高い実行性能を低下させたくないためである。`~~a~~`のような文脈自由構文は容易に追加できるがこうしたマージンを失えばもはや後はない。CommonMarkは小さく単純であるがゆえに正しくいられる象牙の塔であり仕様策定者はこの正しさを失わず正しいままでいたいがために象牙の塔に引きこもり小さな完全性を固持し続けているのである(でなければ何年も隠さず速やかにこの拡張性の欠如を公表して助力を求めていなければならない)。
Securemarkは線形時間で解析不能な前者の解析方法をメモ化によりおおよそ4n以下の最悪計算量に改善しさらに解析時間と解析範囲の局限により一定時間内で解析不能な入力の影響を局限することでこれらの問題を解決している。この解析方法はほとんどの自然な入力に対して1nに近い時間で効率的に動作し、最悪計算量で低速に動作させる少数の機械的攻撃入力に対してもサーバーで多数のユーザーのリクエストに応じるには低速で脆弱性となる可能性があるがクライアントで単一のユーザーの操作に応じるには十分高速であるためクライアントで解析する限り解析の効率または速度が実用上問題となることはない。

### メモ化

一部の文脈依存言語を線形時間で解析できるようになるとしても状態数に応じて複数回走査が必要なこと、このため時間効率が定性的には改善されても定量的には文脈自由言語より数倍悪いこと、オーバーヘッドが有意に大きいことなどから理論上同じ線形時間だとしても実用上文脈自由言語と同等の実行性能にはならず文脈依存言語と文脈自由言語を等価の選択肢にするものではない。

### 最適化

- メモ化したパース結果を複製せず永続データ構造として使用できるようおよび後処理に必要となるノードを収集しやすいようCSTを木構造に置き換える。
- DOM出力の分離によりバックトラックのコストを削減する。

### 標準化

Markdownのように自然言語と自身の文法を分離する汎用構造がないメタ言語は構文を拡張する際に旧構文を破壊する新構文が自然言語の中に潜在せざるをえず後方互換性を保証することが不可能である。
このためCommonMarkは拡張構文を標準化した次期標準仕様との互換性を確保する役には立たずその有用性は構文の拡張を考慮しない範囲での効率的な実装方法の例示およびテストケースの集積による個別実装の支援ならびにその結果としての限定的互換性にとどまる。
さらにCommonMarkは前述の解析時間の制約から拡張性が低く、高度な構文や機能の実装可能性を考慮していないことから拡張仕様において準拠すべき技術的正当性もない。
実際GFMが二種類のフェンスドブロックのセマンティクスを分けずCommonMarkに準拠して単なるエイリアスとして有用な記号を浪費したのは顕著かつ明白な失敗だったと言えよう(Securemarkでは入力文字列の無変換表示(シンタックスハイライト)用と変換表示(拡張構文)用に分かれている)。

よってMarkdownの標準化は後方互換性確保が不可能であることから発展性がなくスナップショット以上の技術的意味を持たない。
MarkdownはGFMのように最初から高機能で完成度の高い拡張不要な独自実装のほうが標準としての互換性を確保でき、構文に曖昧さがない形式言語と異なりまず最小限の標準仕様を策定しのちに拡張していく通常の標準化方法が適さない特殊な言語である。

### CommonMarkの重大な問題点

- 文脈依存(かつ正規表現で解析できない)構文を追加できない(要解析戦略変更)
- 二重リンク(`[<a@b>]()`)を生成する
- 画像のaltがMarkdownとして解釈される
- タブまたは4スペース以上で字下げできない(要構文削除)
- 情報の再利用性が低い
  - 引用に恒等性がない
  - 改行に恒等性がない

### トランスクルージョン

分散的に管理される情報のトランスクルージョンは権利関係の不明瞭さおよびリンク先の消失によりリンク元の情報に欠損が生じるなどの脆さから壊れやすいウェブ上の情報を扱う方法として既存の方法より劣っておりWikipediaのように中央集権的管理を実施できる場合にのみ有用となる。

### Data URI

Data URIは保存および転送容量削減ならびにユーザーおよび管理者双方の集約的管理のためサポートしない。
特に規制および公開レベルの媒体別設定の実現のためテキストとメディアの分離が必須となる。

### HTML

- \<small>: 法的表記を縮小表示すべきでないため削除。
- \<sub>: \<small>の代わりに使用されないよう削除。他の構文との相性も悪い。
- \<sup>: 同上。

### _ emphasis

オートリンクと単語検索と非常に相性が悪いため不採用。
