import { Parser } from '../combinator/data/parser';
import { convert } from '../combinator';
import { define } from 'typed-dom/dom';

export function lineable<P extends Parser<HTMLElement | string>>(parser: P): P;
export function lineable<T extends HTMLElement | string>(parser: Parser<T>): Parser<T> {
  return convert(source => `\r${source}`, parser);
}

export function markInvalid<T extends Element>(
  el: T,
  syntax: string,
  type: string,
  message: string,
): T {
  assert(!message.endsWith('.'));
  return define(el, {
    class: void el.classList.add('invalid'),
    'data-invalid-syntax': syntax,
    'data-invalid-type': type,
    'data-invalid-message': message,
  });
}

export function unmarkInvalid<T extends Element>(el: T): T {
  return define(el, {
    class: void el.classList.remove('invalid'),
    'data-invalid-syntax': null,
    'data-invalid-type': null,
    'data-invalid-message': null,
  });
}

export function stringify(nodes: readonly (HTMLElement | string)[]): string {
  let acc = '';
  for (let i = 0; i < nodes.length; ++i) {
    const node = nodes[i];
    if (typeof node === 'string') {
      assert(!node.includes('\n'));
      acc += node;
    }
    else {
      assert(!node.matches('br') && !node.querySelector('br'));
      // NOTE: Doesn't reflect line breaks.
      acc += node.innerText;
    }
  }
  return acc;
}
